print("import: 'cherrypy'")
import cherrypy

print("import: 'cherrypy.lib'")
import cherrypy.lib

print("import: 'cherrypy.process'")
import cherrypy.process

print("import: 'cherrypy.scaffold'")
import cherrypy.scaffold

print("import: 'cherrypy.test'")
import cherrypy.test

print("import: 'cherrypy.tutorial'")
import cherrypy.tutorial

